#ifndef INCLUDED__B_F__SIMPLE__G_L_U_T__H
#define INCLUDED__B_F__SIMPLE__G_L_U_T__H

/* FILE:        BFsimpleGLUT.h
 * DATE:        5/8/2014
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2014, BitFlow, Inc.
 * DESCRIPTION: A very simple interface for displaying a single grayscale image using FreeGLUT.
 */

#include <CL/opencl.h>

// Initialize BFsimpleGLUT. Call this once, prior to using any other methods.
bool initBFsimpleGLUT                       ( int argc, char *argv[] );

// Prepare the BFsimpleGLUT buffers. To be called after enterBFsimpleGLUT.
bool makeBFsimpleGLUTbuffers                ( const cl_context oclContext
                                            , const unsigned int width
                                            , const unsigned int height
                                            , const unsigned int bitdepth );

// Enter the GLUT main-loop with the given settings. This will return when the user closes the display
// window. Only on thread should call this method at any given time.
void enterBFsimpleGLUT                      ( const bool fullScreen = false);

// Exit the GLUT main-loop on demand, closing any windows.
void exitBFsimpleGLUT                       (void);

// Setup a callback function to be called whenever an ASCII keyboard key is pressed. If this function
// returns true, the display will ignore the keypress internally. If false, the keypress may invoke
// some standard internal behavior.
typedef bool (*BFsimpleGLUTkeyCallback)     (char key);

void setBFsimpleGLUTkeyCallback             (BFsimpleGLUTkeyCallback callback);

// Pass a buffer to BFsimpleGLUT for display (we target 60Hz display; some buffers may be dropped).
// A prior call to enterBFsimpleGLUT should still be running.
cl_int enqueueSetMainBFsimpleGLUT           ( const cl_command_queue oclCmdQueue
                                            , const cl_mem oclBuffer
                                            , const cl_uint numEventsInWaitList
                                            , const cl_event *const eventWaitList
                                            , cl_event *oclEvent );

// Set the overlay text.
void setBFsimpleGLUToverlayText             (const char *const text);

// Return the estimated rate in Hz at which frames are being displayed.
double getBFsimpleGLUTdisplayRateAverage    (void);

#endif // INCLUDED__B_F_D_V_P__SIMPLE__G_L_U_T__H
