/* FILE:        BufferGain.cl
 * DATE:        6/12/2015
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2015, BitFlow, Inc.
 * DESCRIPTION: Implement an OpenCL kernel to perform a buffer gain operation, incrementing every
 *              pixel by a given value, and performing boundary checking. The Global Work Size should
 *              be one quarter the image byte size (i.e., the image WORD size, in 32-bit words).
 */

__kernel void buffer_gain_8bit (__global uchar4 *dst, __global const uchar4 *src, int gainNumerator, int gainDenominator, int offset)
{
    // Determine
    uint tid = get_global_id(0);

    // Get the initial value. We will perform processing as 32-bit ints, to prevent overflow.
    int4 dProc = convert_int4(src[tid]);
    
    // Apply the gain operation.
    dProc = dProc * gainNumerator / gainDenominator + offset;
    
    // Set the result to the output, clamping the results to the valid range.
    dst[tid] = convert_uchar4_sat(dProc);
}

__kernel void buffer_gain_16bit (__global ushort4 *dst, __global const ushort4 *src, int gainNumerator, int gainDenominator, int offset)
{
    // Determine
    uint tid = get_global_id(0);

    // Get the initial value. We will perform processing as 32-bit ints, to prevent overflow.
    long4 dProc = convert_long4(src[tid]);
    
    // Apply the gain operation.
    dProc = dProc * (long)gainNumerator / (long)gainDenominator + (long)offset;
    
    // Set the result to the output, clamping the results to the valid range.
    dst[tid] = convert_ushort4_sat(dProc);
}
